@@ -26,7 +26,7 @@ class QPluginLoader;
 class KPluginMetaDataPrivate;
 class KAboutPerson;
 class QObject;
-
+class QStaticPlugin;
 /**
  * @class KPluginMetaData kpluginmetadata.h KPluginMetaData
  *
@@ -52,6 +52,7 @@ class QObject;
  * Id                 | pluginId()           | string
  * Version            | version()            | string
  * Website            | website()            | string
+ * BugReportUrl       | bugReportUrl()       | string
  * EnabledByDefault   | isEnabledByDefault() | bool
  * ServiceTypes       | serviceTypes()       | string array
  * MimeTypes          | mimeTypes()          | string array
@@ -107,16 +108,30 @@ class KCOREADDONS_EXPORT KPluginMetaData
     Q_PROPERTY(QString pluginId READ pluginId CONSTANT)
     Q_PROPERTY(QString version READ version CONSTANT)
     Q_PROPERTY(QString website READ website CONSTANT)
+    Q_PROPERTY(QString bugReportUrl READ bugReportUrl CONSTANT)
 #if KCOREADDONS_ENABLE_DEPRECATED_SINCE(5, 79)
     Q_PROPERTY(QStringList dependencies READ dependencies CONSTANT)
 #endif
+#if KCOREADDONS_ENABLE_DEPRECATED_SINCE(5, 89)
     Q_PROPERTY(QStringList serviceTypes READ serviceTypes CONSTANT)
+#endif
     Q_PROPERTY(QStringList mimeTypes READ mimeTypes CONSTANT)
     Q_PROPERTY(QStringList formFactors READ formFactors CONSTANT)
     Q_PROPERTY(bool isEnabledByDefault READ isEnabledByDefault CONSTANT)
+#if KCOREADDONS_ENABLE_DEPRECATED_SINCE(5, 104)
     Q_PROPERTY(int initialPreference READ isEnabledByDefault CONSTANT)
+#endif
 
 public:
+    /**
+     * Options for creating a KPluginMetaData object.
+     * @since 5.91
+     */
+    enum KPluginMetaDataOption {
+        DoNotAllowEmptyMetaData, /// Plugins with empty metaData are considered invalid
+        AllowEmptyMetaData, /// Plugins with empty metaData are considered valid
+    };
+
     /** Creates an invalid KPluginMetaData instance */
     KPluginMetaData();
 
@@ -147,11 +162,21 @@ public:
      *
      * If @p file ends with .json, the file will be loaded as the QJsonObject metadata.
      *
+     * @note Using this constructor for metadata files is deprecated..
+     * Use KPluginMetaData::fromDesktopFile or KPluginMetaData::fromJsonFile instead.
+     *
      * @see QPluginLoader::setFileName()
      * @see KPluginMetaData::fromDesktopFile()
+     * @see KPluginMetaData::fromJsonFile()
      */
     KPluginMetaData(const QString &file);
 
+    /**
+     * Overload which takes an option parameter that gets used when creating the KPluginMetaData instances
+     * @since 5.91
+     */
+    KPluginMetaData(const QString &file, KPluginMetaDataOption option);
+
     /**
      * Creates a KPluginMetaData from a QJsonObject holding the metadata and a file name
      * This can be used if the data is not retrieved from a Qt C++ plugin library but from some
@@ -174,6 +199,17 @@ public:
      */
     KPluginMetaData(const QJsonObject &metaData, const QString &pluginFile, const QString &metaDataFile);
 
+    /*
+     * Constructs a KPluginMetaData from the static plugin.
+     * If it does not have any meta data the @p metaData value is used
+     *
+     * @see KPluginFactory::loadFactory
+     * @see KPluginFactory::instantiatePlugin
+     *
+     * @since 5.89
+     */
+    KPluginMetaData(QStaticPlugin plugin, const QJsonObject &metaData = {});
+
     /**
      * Copy contructor
      */
@@ -187,8 +223,9 @@ public:
      */
     ~KPluginMetaData();
 
+#if KCOREADDONS_ENABLE_DEPRECATED_SINCE(5, 92)
     /**
-     * Load a KPluginMetaData instace from a .desktop file. Unlike the constructor which takes
+     * Load a KPluginMetaData instance from a .desktop file. Unlike the constructor which takes
      * a single file parameter this method allows you to specify which service type files should
      * be parsed to determine the correct type for a given .desktop property.
      * This ensures that a e.g. comma-separated string list field in the .desktop file will correctly
@@ -204,8 +241,20 @@ public:
      * will be read as the JSON string type.
      *
      * @since 5.16
+     * @deprecated Since 5.92, use json files or embedded json metadata directly.
      */
+    KCOREADDONS_DEPRECATED_VERSION(5, 92, "Use json files or embedded json metadata directly")
     static KPluginMetaData fromDesktopFile(const QString &file, const QStringList &serviceTypes = QStringList());
+#endif
+
+    /**
+     * Load a KPluginMetaData instance from a .json file. Unlike the constructor with a single file argument,
+     * this ensure that only JSON format plugins are loaded and any other type is rejected.
+     *
+     * @param jsonFile the .json file to load
+     * @since 5.91
+     */
+    static KPluginMetaData fromJsonFile(const QString &jsonFile);
 
     /**
      * @param directory The directory to search for plugins. If a relative path is given for @p directory,
@@ -233,6 +282,11 @@ public:
      */
     static QVector<KPluginMetaData> findPlugins(const QString &directory, std::function<bool(const KPluginMetaData &)> filter = {});
 
+    /**
+     * @since 5.91
+     */
+    static QVector<KPluginMetaData> findPlugins(const QString &directory, std::function<bool(const KPluginMetaData &)> filter, KPluginMetaDataOption option);
+
     /**
      * @return whether this object holds valid information about a plugin.
      * If this is @c true pluginId() will return a non-empty string.
@@ -372,6 +426,12 @@ public:
      */
     QString website() const;
 
+    /**
+     * @return the website where people can report a bug found in this plugin
+     * @since 5.99
+     */
+    QString bugReportUrl() const;
+
 #if KCOREADDONS_ENABLE_DEPRECATED_SINCE(5, 79)
     /**
      * @return a list of plugins that this plugin depends on so that it can function properly
@@ -382,6 +442,7 @@ public:
     QStringList dependencies() const;
 #endif
 
+#if KCOREADDONS_ENABLE_DEPRECATED_SINCE(5, 89)
     /**
      * Returns the service types that this plugin implements.
      *
@@ -393,8 +454,11 @@ public:
      * @note Unlike KService this does not contain the MIME types. To get the handled MIME types
      * use the KPluginMetaData::mimeTypes() function.
      * @return a list of service types this plugin implements (e.g. "Plasma/DataEngine")
+     * @dprecated Since 5.89, use dedicated plugin namespaces instead to filter plugins of a specific type
      */
+    KCOREADDONS_DEPRECATED_VERSION(5, 89, "See API docs")
     QStringList serviceTypes() const;
+#endif
 
     /**
      * @return a list of MIME types this plugin can handle (e.g. "application/pdf", "image/png", etc.)
@@ -425,14 +489,34 @@ public:
      */
     bool isEnabledByDefault() const;
 
+#if KCOREADDONS_ENABLE_DEPRECATED_SINCE(5, 104)
     /**
      * @return the initial preference of the plugin.
      * This is the preference to associate with this plugin initially (before
      * the user has had any chance to define preferences for it).
      * Higher values indicate stronger preference.
      * @since 5.67
+     * @deprecated Since 5.104, this feature is only used in KParts, read the key manually if needed
      */
+    KCOREADDONS_DEPRECATED_VERSION(5, 104, "This feature is only used in KParts, read the key manually if needed")
     int initialPreference() const;
+#endif
+
+    /**
+     * Returns @c true if the plugin is enabled in @p config, otherwise returns isEnabledByDefault().
+     * This can be used in conjunctionwith KPluginWidget/KPluginSelector.
+     *
+     * The @p config param should be a KConfigGroup object, because KCoreAddons can not depend
+     * on KConfig directly, this parameter is a template.
+     * @param config KConfigGroup where the enabled state is stored
+     * @since 5.89
+     */
+    template<typename T>
+    bool isEnabled(const T &config) const
+    {
+        Q_ASSERT(config.isValid());
+        return config.readEntry(pluginId() + QLatin1String("Enabled"), isEnabledByDefault());
+    }
 
     /**
      * @return the value for @p key from the metadata or @p defaultValue if the key does not exist
@@ -469,9 +553,9 @@ public:
      */
     int value(const QString &key, int defaultValue) const;
 
-    /** @return the value for @p key inside @p jo as a string list. If the type of @p key is string, a list with containing
-     * just that string will be returned, if it is an array the list will contain one entry for each array member.
-     * If the key cannot be found an empty list will be returned.
+    /** @return the value for @p key from the metadata or @p defaultValue if the key does not exist.
+     * If the type of @p key is string, a list containing just that string will be returned.
+     * If the type is array, the list will contain one entry for each array member.
      * @overload
      * @since 5.88
      */
@@ -524,21 +608,39 @@ public:
         return !(*this == other);
     }
 
+    /**
+     * @note for loading plugin the plugin independently of it being static/dynamic
+     * use @ref KPluginFactory::loadFactory or @ref KPluginFactory::instantiatePlugin.
+     * @return true if the instance represents a static plugin
+     * @since 5.89
+     */
+    bool isStaticPlugin() const;
+
 private:
     QJsonObject rootObject() const;
+#if KCOREADDONS_BUILD_DEPRECATED_SINCE(5, 92)
     void loadFromDesktopFile(const QString &file, const QStringList &serviceTypes);
+#endif
+    void loadFromJsonFile(const QString &file);
 
 private:
     QVariantList authorsVariant() const;
     QVariantList translatorsVariant() const;
     QVariantList otherContributorsVariant() const;
+    QStaticPlugin staticPlugin() const;
+    QString requestedFileName() const;
 
     QJsonObject m_metaData;
     QString m_fileName;
     QExplicitlySharedDataPointer<KPluginMetaDataPrivate> d; // for future binary compatible extensions
+    friend class KPluginFactory;
 };
 
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+inline size_t qHash(const KPluginMetaData &md, size_t seed)
+#else
 inline uint qHash(const KPluginMetaData &md, uint seed)
+#endif
 {
     return qHash(md.pluginId(), seed);
 }
